
package com.xdf.creative.module.controller;

import com.xdf.creative.base.controller.BaseController;
import com.xdf.creative.base.params.creative.SysUserQueryParam;
import com.xdf.creative.base.params.creative.UpdatePasswordParam;
import com.xdf.creative.base.params.creative.UpdateUserParam;
import com.xdf.creative.base.vo.creative.SysUserQueryVo;
import com.xdf.creative.module.entity.SysUser;
import com.xdf.creative.module.service.SysUserService;
import com.xdf.creative.support.auth.LoginParam;
import com.xdf.creative.support.auth.LoginSysUserTokenVo;
import com.xdf.creative.util.page.ApiResult;
import com.xdf.creative.util.page.PageTool;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;

/**
 * <pre>
 * 系统用户 前端控制器
 * </pre>
 *
 * @author DeanYe
 * @since 2019-10-24
 */
@Slf4j
@RestController
@RequestMapping("/sysUser")
@Api("系统用户 API")
public class SysUserController extends BaseController {

    @Autowired
    private SysUserService sysUserService;


    @PostMapping("/login")
    @ApiOperation(value = "登陆", notes = "系统用户登陆", response = ApiResult.class)
    public ApiResult login(@Valid @RequestBody LoginParam loginParam, HttpServletResponse response) throws Exception {
        LoginSysUserTokenVo loginSysUserTokenVo = sysUserService.login(loginParam);
        // 设置token响应头
      //  response.setHeader(JwtTokenUtil.getTokenName(), loginSysUserTokenVo.getToken());
        return ApiResult.ok(loginSysUserTokenVo, "登陆成功");
    }


    /**
     * 添加系统用户
     */
    @PostMapping("/add")
    @ApiOperation(value = "添加SysUser对象", notes = "添加系统用户", response = ApiResult.class)
    public ApiResult<Boolean> addSysUser(@Valid @RequestBody SysUser sysUser) throws Exception {
        boolean flag = sysUserService.saveSysUser(sysUser);
        return ApiResult.result(flag);
    }

    /**
     * 修改系统用户
     */
    @PostMapping("/update")
    @ApiOperation(value = "修改SysUser对象", notes = "修改系统用户", response = ApiResult.class)
    public ApiResult<Boolean> updateSysUser(@Valid @RequestBody UpdateUserParam updateUserParam) throws Exception {
        boolean flag = sysUserService.updateSysUser(updateUserParam);
        return ApiResult.result(flag);
    }

    /**
     * 删除系统用户
     */
    @RequestMapping(value = "/delete", method = {RequestMethod.GET, RequestMethod.POST})
    @ApiOperation(value = "删除SysUser对象", notes = "删除系统用户", response = ApiResult.class)
    public ApiResult<Boolean> deleteSysUser(Long id) throws Exception {
        boolean flag = sysUserService.deleteSysUser(id);
        return ApiResult.result(flag);
    }

    /**
     * 获取系统用户
     */
    @GetMapping("/getSysUserById")
    @ApiOperation(value = "获取SysUser对象详情", notes = "查看系统用户", response = SysUserQueryVo.class)
    public ApiResult<SysUserQueryVo> getSysUser(Long id) throws Exception {
        SysUserQueryVo sysUserQueryVo = sysUserService.getSysUserById(id);
        return ApiResult.ok(sysUserQueryVo);
    }

    /**
     * 获取系统用户
     */
    @GetMapping("/getSysUserByUserName")
    @ApiOperation(value = "获取SysUser对象详情", notes = "查看系统用户", response = SysUserQueryVo.class)
    public ApiResult<SysUser> getSysUserByUserName(@RequestParam String username) throws Exception {
        SysUser sysUser = sysUserService.getSysUserByUserName(username);
        return ApiResult.ok(sysUser);
    }

    /**
     * 系统用户分页列表
     */
    @PostMapping("/getPageList")
    @ApiOperation(value = "获取SysUser分页列表", notes = "系统用户分页列表", response = SysUserQueryVo.class)
    public ApiResult<PageTool<SysUserQueryVo>> getSysUserPageList(@Valid @RequestBody SysUserQueryParam sysUserQueryParam) throws Exception {
        PageTool<SysUserQueryVo> pageList = sysUserService.getSysUserPageList(sysUserQueryParam);
        return ApiResult.ok(pageList);
    }

    /**
     * 修改密码
     */
    @PostMapping("/updatePassword")
    @ApiOperation(value = "修改密码", notes = "修改密码", response = ApiResult.class)
    public ApiResult<Boolean> updatePassword(@Valid @RequestBody UpdatePasswordParam updatePasswordParam) throws Exception {
        boolean flag = sysUserService.updatePassword(updatePasswordParam);
        return ApiResult.result(flag);
    }
}

