
package com.xdf.creative.module.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xdf.creative.base.params.creative.SysPermissionQueryParam;
import com.xdf.creative.base.service.impl.BaseServiceImpl;
import com.xdf.creative.base.vo.creative.SysPermissionQueryVo;
import com.xdf.creative.enums.StateEnum;
import com.xdf.creative.module.entity.SysPermission;
import com.xdf.creative.module.entity.SysRolePermission;
import com.xdf.creative.module.mapper.SysPermissionMapper;
import com.xdf.creative.module.service.SysPermissionService;
import com.xdf.creative.module.service.SysRolePermissionService;
import com.xdf.creative.support.exception.BusinessException;
import com.xdf.creative.util.StringUtil;
import com.xdf.creative.util.page.PageTool;
import com.xdf.creative.util.tree.TreeConfig;
import com.xdf.creative.util.tree.Utils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.Serializable;
import java.util.*;


/**
 * <pre>
 * 系统权限 服务实现类
 * </pre>
 *
 * @author DeanYe
 * @since 2019-10-25
 */
@Slf4j
@Service
public class SysPermissionServiceImpl extends BaseServiceImpl<SysPermissionMapper, SysPermission> implements SysPermissionService {

    @Autowired
    private SysPermissionMapper sysPermissionMapper;

    @Resource
    private SysRolePermissionService sysRolePermissionService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean saveSysPermission(SysPermission sysPermission) throws Exception {
        sysPermission.setId(null);
        return super.save(sysPermission);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateSysPermission(SysPermission sysPermission) throws Exception {
        // 获取权限
        SysPermission updateSysPermission = getById(sysPermission.getResourceId());
        if (updateSysPermission == null) {
            throw new BusinessException("权限不存在");
        }

        // 指定需改的字段
        updateSysPermission.setParentResourceId(sysPermission.getParentResourceId())
                .setLevel(sysPermission.getLevel())
                .setName(sysPermission.getName())
                .setState(sysPermission.getState())
                .setCode(sysPermission.getCode())
                .setIcon(sysPermission.getIcon())
                .setRemark(sysPermission.getRemark())
                .setSort(sysPermission.getSort())
                .setType(sysPermission.getType())
                .setUrl(sysPermission.getUrl())
                .setUpdateTime(new Date());
        return super.updateById(updateSysPermission);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean deleteSysPermission(String resourceId) throws Exception {
        boolean isExists = sysRolePermissionService.isExistsByPermissionId(resourceId);
        if (isExists) {
            throw new BusinessException("该权限存在角色关联关系，不能删除");
        }
        return super.removeById(resourceId);
    }

    @Override
    public SysPermissionQueryVo getSysPermissionById(Serializable id) throws Exception {
        return sysPermissionMapper.getSysPermissionById(id);
    }

    @Override
    public PageTool<SysPermissionQueryVo> getSysPermissionPageList(SysPermissionQueryParam sysPermissionQueryParam) throws Exception {
        Page page = setPageParam(sysPermissionQueryParam);
        IPage<SysPermissionQueryVo> iPage = sysPermissionMapper.getSysPermissionPageList(page, sysPermissionQueryParam);
        return new PageTool(iPage);
    }

    @Override
    public boolean isExistsByPermissionIds(List<String> permissionIds) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.in("resourceId", permissionIds);
        return sysPermissionMapper.selectCount(queryWrapper).intValue() == permissionIds.size();
    }

    @Override
    public List<SysPermission> getAllMenuList() throws Exception {
        SysPermission sysPermission = new SysPermission().setState(StateEnum.ENABLE.ordinal());
        // 获取所有已启用的权限列表
        return sysPermissionMapper.selectList(new QueryWrapper(sysPermission));
    }

    /**
     * 获取角色授权资源树
     *
     * @param userId
     * @return
     * @throws Exception
     */
    @Override
    public List<Object> impowerResourceTreeList(Long userId) throws Exception {
        return sysPermissionMapper.impowerResourceTreeList(userId);
    }


    @Override
    public List<SysPermission> getMenuListByUserId(Long userId) throws Exception {
        return sysPermissionMapper.getMenuListByUserId(userId);
    }

    @Override
    public List<Object> getTreeMenusByUserId(Long userId) throws Exception {
        // 获取所有已启用的权限列表
        List<Object> list = sysPermissionMapper.getMenusByUserId(userId);
        TreeConfig config = new TreeConfig("resourceId", "parentResourceId", "parentResourceId", "01", "");
        List<Object> objectList = Utils.tree(list, config);
        return objectList;
    }


    @Override
    public Set<String> getPermissionUrlByUserId(Long userId) throws Exception {
        List<String> permissionUrls = sysPermissionMapper.getPermissionUrlByUserId(userId);

        HashSet<String> stringHashSet = new HashSet<>();
        permissionUrls.stream().forEach(s -> {
            if (StringUtil.isNotEmpty(s)) {
                if (s.contains(";")) {
                    String[] url = s.split(s);
                    for (String s1 : url) {
                        stringHashSet.add(s1);
                    }
                } else {
                    stringHashSet.add(s);
                }
            }
        });
        return stringHashSet;
    }

    @Override
    public Boolean setPermissionToRoleByUserId(Long userId, Long roleId) throws Exception {
        QueryWrapper queryWrapper = new QueryWrapper();
        List<SysPermission> list = sysPermissionMapper.selectList(queryWrapper);

        List<SysRolePermission> sysRolePermissions = new ArrayList<>();
        list.stream().forEach(sysPermission -> {
            SysRolePermission sysRolePermission = new SysRolePermission();
            String resourceId = sysPermission.getResourceId();
            sysRolePermission.setPermissionId(resourceId);
            sysRolePermission.setRoleId(roleId);
            sysRolePermission.setRemark(sysPermission.getName());
            sysRolePermissions.add(sysRolePermission);
        });
        boolean isSucceed = sysRolePermissionService.saveBatch(sysRolePermissions, 20);
        return isSucceed;
    }

    @Override
    public List<String> getSkipAuthUrls() throws Exception {
        List<String> skipAuthUrls = sysPermissionMapper.getSkipAuthUrls();
        return skipAuthUrls;
    }
}
