
package com.xdf.creative.module.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xdf.creative.base.params.creative.IsEnableSysOrganizeParam;
import com.xdf.creative.base.params.creative.SysUserQueryParam;
import com.xdf.creative.base.params.creative.UpdatePasswordParam;
import com.xdf.creative.base.params.creative.UpdateUserParam;
import com.xdf.creative.base.service.impl.BaseServiceImpl;
import com.xdf.creative.base.vo.creative.SysOrganizeQueryVo;
import com.xdf.creative.base.vo.creative.SysRoleQueryVo;
import com.xdf.creative.base.vo.creative.SysUserQueryVo;
import com.xdf.creative.enums.ApiCode;
import com.xdf.creative.enums.StateEnum;
import com.xdf.creative.module.entity.SysUser;
import com.xdf.creative.module.entity.SysUserRole;
import com.xdf.creative.module.mapper.SysUserMapper;
import com.xdf.creative.module.service.SysOrganizeService;
import com.xdf.creative.module.service.SysRoleService;
import com.xdf.creative.module.service.SysUserRoleService;
import com.xdf.creative.module.service.SysUserService;
import com.xdf.creative.shiro.jwt.JwtProperties;
import com.xdf.creative.shiro.jwt.JwtToken;
import com.xdf.creative.shiro.jwt.JwtUtil;
import com.xdf.creative.shiro.jwt.SaltUtil;
import com.xdf.creative.support.auth.LoginParam;
import com.xdf.creative.support.auth.LoginSysUserTokenVo;
import com.xdf.creative.support.auth.LoginSysUserVo;
import com.xdf.creative.support.exception.BusinessException;
import com.xdf.creative.support.exception.SysLoginException;
import com.xdf.creative.util.PasswordUtil;
import com.xdf.creative.util.StringUtil;
import com.xdf.creative.util.UUIDUtil;
import com.xdf.creative.util.convert.SysUserConvert;
import com.xdf.creative.util.page.PageTool;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import cn.hutool.core.collection.CollectionUtil;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.authc.UsernamePasswordToken;
import org.apache.shiro.subject.Subject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.Serializable;
import java.time.Duration;
import java.util.*;


/**
 * <pre>
 * 系统用户 服务实现类
 * </pre>
 *
 * @author DeanYe
 * @since 2019-10-24
 */
@Api
@Slf4j
@Service
public class SysUserServiceImpl extends BaseServiceImpl<SysUserMapper, SysUser> implements SysUserService {

    @Autowired
    private SysUserMapper sysUserMapper;

    @Lazy
    @Autowired
    private SysRoleService sysRoleService;

    @Lazy
    @Autowired
    private SysOrganizeService sysOrganizeService;

    @Lazy
    @Autowired
    private SysUserRoleService sysUserRoleService;

    @Autowired
    private JwtProperties jwtProperties;


    @Override
    public LoginSysUserTokenVo login(LoginParam loginParam) throws Exception {
        // 校验验证码
        String userName = loginParam.getUsername();
        SysUser sysUser = getSysUserByUserName(userName);
        if (sysUser == null) {
            log.error("登陆失败，用户名不正确,loginParam:{}", loginParam);
            throw new SysLoginException(ApiCode.USERNAME_ERROR);
        }
        if (StateEnum.DISABLE.getKey().equals(sysUser.getState())) {
            log.debug("登陆失败，用户不可用,loginParam:{}", loginParam);
            throw new SysLoginException(ApiCode.USER_DISABLE);
        }

        String encryptPassword = PasswordUtil.encrypt(loginParam.getPassword(), sysUser.getSalt());
        if (!encryptPassword.equals(sysUser.getPassword())) {
            throw new SysLoginException(ApiCode.PASSWORD_ERROR);
        }


        SysOrganizeQueryVo sysOrganizeQueryVo = sysOrganizeService.getSysOrganizeById(sysUser.getOrganizeId());
        if (sysOrganizeQueryVo == null) {
            throw new BusinessException("组织不存在");
        }
        if (!StateEnum.ENABLE.getKey().equals(sysOrganizeQueryVo.getState())) {
            throw new BusinessException("组织已禁用");
        }
        // 将系统用户对象转换成登陆用户对象
        LoginSysUserVo loginSysUserVo = SysUserConvert.INSTANCE.sysUserToLoginSysUserVo(sysUser);
        loginSysUserVo.setOrganizeId(sysOrganizeQueryVo.getId());

        //获取角色id、名称、code
        HashSet<Long> roleIdSet = new HashSet<>();
        HashSet<String> roleNameSet = new HashSet<>();
        HashSet<String> roleCodeSet = new HashSet<>();

        List<SysRoleQueryVo> sysRoleQueryVoList = sysRoleService.getUserRoleByUserId(sysUser.getId());
        if (CollectionUtil.isEmpty(sysRoleQueryVoList)) {
            throw new BusinessException("角色不存在");
        }
        sysRoleQueryVoList.stream().forEach(sysRoleQueryVo -> {
            roleIdSet.add(sysRoleQueryVo.getId());
            roleNameSet.add(sysRoleQueryVo.getName());
            roleCodeSet.add(sysRoleQueryVo.getCode());
        });
        loginSysUserVo.setRoleId(roleIdSet)
                .setRoleName(roleNameSet)
                .setRoleCode(roleCodeSet);
        //获取角色url

        // 获取数据库中保存的盐值
        String newSalt = SaltUtil.getSalt(sysUser.getSalt(), jwtProperties);
        // 生成token字符串并返回
        Long expireSecond = jwtProperties.getExpireSecond();
        String token = JwtUtil.generateToken(sysUser.getId(), userName, sysUser.getOrganizeId(),sysUser.getUserType(), newSalt, Duration.ofSeconds(expireSecond));
        log.debug("token:{}", token);
        // 创建AuthenticationToken
        JwtToken jwtToken = new JwtToken(token);
        Subject subject = SecurityUtils.getSubject();
        //进行验证，这里可以捕获异常，然后返回对应信息
        subject.login(jwtToken);

        // 返回token和登陆用户信息对象
        LoginSysUserTokenVo loginSysUserTokenVo = new LoginSysUserTokenVo();
        loginSysUserTokenVo.setToken(token);
        loginSysUserTokenVo.setLoginSysUserVo(loginSysUserVo);
        return loginSysUserTokenVo;
    }




    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean saveSysUser(SysUser sysUser) throws Exception {
        // 校验用户名是否存在
        boolean isExists = isExistsByUsername(sysUser.getUsername());
        if (isExists) {
            throw new BusinessException("用户名已存在");
        }
        // 校验组织和角色
        checkOrganizeAndRole(sysUser.getOrganizeId(), sysUser.getRoleId());
       // Long userId = getUserId();//获取当前用户id
        // 生成盐值
        String salt = UUIDUtil.getUUID();
        sysUser.setSalt(salt);
        sysUser.setId(null);
        //sysUser.setCreateUserId(userId);

        // 密码加密
        String newPassword = PasswordUtil.encrypt(sysUser.getPassword(), salt);
        sysUser.setPassword(newPassword);

        // 保存系统用户
        return super.save(sysUser);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateSysUser(UpdateUserParam updateUserParam) throws Exception {

        // 获取系统用户
        SysUser updateSysUser = getById(updateUserParam.getId());
        if (updateSysUser == null) {
            throw new BusinessException("修改的用户不存在");
        }
        // 校验组织和角色
        checkOrganizeAndRole(updateUserParam.getOrganizeId(), updateUserParam.getRoleId());

        //String jsonStr = (String) redisTemplate.opsForValue().get(String.format(CommonRedisKey.LOGIN_USER, updateSysUser.getUsername()));
        //LoginSysUserRedisVo loginSysUserRedisVo = JSONObject.parseObject(jsonStr, LoginSysUserRedisVo.class);
        //权限修改
        Set<Long> roleIds = updateUserParam.getRoleId();
        if (1==1){//CollectionUtil.isNotEmpty(roleIds)) {
            QueryWrapper<SysUserRole> queryWrapper = new QueryWrapper();
            queryWrapper.eq("user_id", updateUserParam.getId());
            sysUserRoleService.remove(queryWrapper);

            List<SysUserRole> sysUserRoles = new ArrayList<>();
            for (Long roleId : roleIds) {
                SysUserRole sysUserRole = new SysUserRole();
                sysUserRole.setRoleId(roleId);
                sysUserRole.setUserId(updateUserParam.getId());
                sysUserRoles.add(sysUserRole);
            }
            //sysUserRoleService.saveBatch(sysUserRoles);
            Set<Long> roleSet = new HashSet<>();
            Set<String> roleNameSet = new HashSet<>();
            List<SysRoleQueryVo> sysRoleQueryVos = sysRoleService.getUserRoleByUserId(null);//getUserId());
            sysRoleQueryVos.stream().forEach(sysRoleQueryVo -> {
                roleSet.add(sysRoleQueryVo.getId());
                roleNameSet.add(sysRoleQueryVo.getName());
            });
//            loginSysUserRedisVo.setRoleId(roleSet);
//            loginSysUserRedisVo.setRoleName(roleNameSet);
        }

        // 修改系统用户
        updateSysUser.setNickname(updateUserParam.getNickname())
                .setPhone(updateUserParam.getPhone())
                .setGender(updateUserParam.getGender())
                .setRemark(updateUserParam.getRemark())
                .setState(updateUserParam.getState())
                .setOrganizeId(updateUserParam.getOrganizeId())
                .setUpdateTime(new Date());
        super.updateById(updateSysUser);

//        if (StringUtil.isNotEmpty(updateUserParam.getNickname())) {
//            loginSysUserRedisVo.setNickname(updateUserParam.getNickname());
//        }
//        if (StringUtil.isNotEmpty(updateUserParam.getPhone())) {
//            loginSysUserRedisVo.setPhone(updateUserParam.getPhone());
//        }
//        if (updateUserParam.getGender() != null) {
//            loginSysUserRedisVo.setGender(updateUserParam.getGender());
//        }
//        if (updateUserParam.getState() != null) {
//            loginSysUserRedisVo.setState(updateUserParam.getState());
//        }
//        if (StringUtil.longIsNotEmpty(updateUserParam.getOrganizeId())) {
//            loginSysUserRedisVo.setOrgId(updateUserParam.getOrganizeId());
//        }
//
//
//        Long expireDuration = redisTemplate.getExpire(String.format(CommonRedisKey.LOGIN_USER, updateSysUser.getUsername()));
//        redisTemplate.delete(String.format(CommonRedisKey.LOGIN_USER, updateSysUser.getUsername()));
//        String key = String.format(CommonRedisKey.LOGIN_USER, updateSysUser.getUsername());
//        String val = JSON.toJSONString(loginSysUserRedisVo);
//        redisUtil.set(key, val, expireDuration);
        return true;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean deleteSysUser(Long id) throws Exception {
        return super.removeById(id);
    }

    @Override
    public SysUserQueryVo getSysUserById(Serializable id) throws Exception {
        return sysUserMapper.getSysUserById(id);
    }

    @Override
    public PageTool<SysUserQueryVo> getSysUserPageList(SysUserQueryParam sysUserQueryParam) throws Exception {
        Page page = setPageParam(sysUserQueryParam);
        IPage<SysUserQueryVo> iPage = sysUserMapper.getSysUserPageList(page, sysUserQueryParam);
        return new PageTool(iPage);
    }

    @Override
    public boolean isExistsByUsername(String username) throws Exception {
        SysUser selectSysUser = new SysUser().setUsername(username);
        return sysUserMapper.selectCount(new QueryWrapper<>(selectSysUser)) > 0;
    }

    @Override
    public void checkOrganizeAndRole(Long organizeId, Set<Long> roleId) throws Exception {
        // 校验组织是否存在并且可用
        if (StringUtil.longIsNotEmpty(organizeId)) {
            boolean isEnableSysOrganize = sysOrganizeService.isEnableSysOrganize(new IsEnableSysOrganizeParam(null, organizeId));//getUserId(), organizeId));
            if (!isEnableSysOrganize) {
                throw new BusinessException("该组织不存在或已禁用");
            }
        }
        // 校验角色是否存在并且可用
        if (1==1){//CollectionUtil.isNotEmpty(roleId)) {
            boolean isEnable = sysRoleService.isEnableSysRole(roleId);
            if (!isEnable) {
                throw new BusinessException("该角色不存在或已禁用");
            }
        }

    }

    @Override
    public boolean isExistsSysUserByRoleId(Long roleId) throws Exception {
        SysUser sysUser = new SysUser()
                .setState(StateEnum.ENABLE.ordinal());
        return sysUserMapper.selectCount(new QueryWrapper(sysUser)) > 0;
    }

    @Override
    public boolean updatePassword(UpdatePasswordParam updatePasswordParam) throws Exception {
        String oldPassword = updatePasswordParam.getOldPassword();
        String newPassword = updatePasswordParam.getNewPassword();
        String confirmPassword = updatePasswordParam.getConfirmPassword();
        if (!newPassword.equals(confirmPassword)) {
            throw new BusinessException("两次输入的密码不一致");
        }
        if (newPassword.equals(oldPassword)) {
            throw new BusinessException("新密码和旧密码不能一致");
        }

        // 判断原密码是否正确
        SysUser sysUser = null;//getById(updatePasswordParam.getUserId());
        if (sysUser == null) {
            throw new BusinessException("用户不存在");
        }
        if (StateEnum.DISABLE.getKey().equals(sysUser.getState())) {
            throw new BusinessException("用户已禁用");
        }
        // 密码加密处理
        String salt = sysUser.getSalt();
        String encryptOldPassword = PasswordUtil.encrypt(oldPassword, salt);
        if (!sysUser.getPassword().equals(encryptOldPassword)) {
            throw new BusinessException("原密码错误");
        }
        // 新密码加密
        String encryptNewPassword = PasswordUtil.encrypt(newPassword, salt);

        // 修改密码
        sysUser.setPassword(encryptNewPassword)
                .setUpdateTime(new Date());
        return updateById(sysUser);
    }

    @Override
    public boolean updateSysUserHead(Long id, String headPath) throws Exception {
        SysUser sysUser = new SysUser()
                .setId(id)
                .setHead(headPath);
        return updateById(sysUser);
    }

    @Override
    public SysUser getSysUserByUserName(String username) throws Exception {
        return sysUserMapper.getSysUserByUserName(username);
    }
}
